use nom::bytes::complete::is_not;
use nom::bytes::complete::tag;
use nom::combinator::recognize;
use nom::multi::many0;
use nom::sequence::tuple;

use super::keyword::affiliated_keyword;
use super::org_source::OrgSource;
use super::util::get_name;
use super::util::org_line_ending;
use crate::context::RefContext;
use crate::error::Res;
use crate::parser::util::get_consumed;
use crate::parser::util::start_of_line;
use crate::types::DiarySexp;

#[cfg_attr(feature = "tracing", tracing::instrument(ret, level = "debug"))]
pub(crate) fn diary_sexp<'b, 'g, 'r, 's>(
    _context: RefContext<'b, 'g, 'r, 's>,
    input: OrgSource<'s>,
) -> Res<OrgSource<'s>, DiarySexp<'s>> {
    let (input, affiliated_keywords) = many0(affiliated_keyword)(input)?;
    start_of_line(input)?;
    let (remaining, value) = recognize(tuple((tag("%%("), is_not("\r\n"))))(input)?;
    let (remaining, _eol) = org_line_ending(remaining)?;

    let source = get_consumed(input, remaining);
    Ok((
        remaining,
        DiarySexp {
            source: source.into(),
            name: get_name(&affiliated_keywords),
            value: Into::<&str>::into(value),
        },
    ))
}

#[cfg_attr(feature = "tracing", tracing::instrument(ret, level = "debug"))]
pub(crate) fn detect_diary_sexp<'s>(input: OrgSource<'s>) -> Res<OrgSource<'s>, ()> {
    let (input, _) = many0(affiliated_keyword)(input)?;
    tuple((start_of_line, tag("%%(")))(input)?;
    Ok((input, ()))
}
